from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.requirements import Prefix, Boilerplate, LogicalExpression, OccuringFunctionalityP2, StateValueConstraint, OccuringFunctionalityForLogicalExpression
from reqman.apps.reqtool.rest_api.serializers.requirements import PrefixSerializer, BoilerplateSerializer, LogicalExpressionSerializer, OccuringFunctionalityP2Serializer,StateValueConstraintSerializer, OccuringFunctionalityForLogicalExpressionSerializer
from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

from django.shortcuts import get_object_or_404

class BoilerplateListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateSerializer
    queryset = Boilerplate.objects.all()

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class BoilerplateDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly,
                      IsOwnerOrReadOnly]
    serializer_class = BoilerplateSerializer
    queryset = Boilerplate.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        #print(serializer.validated_data['id'])
        #if not Prefix.objects.filter(boilerplate = serializer.validated_data['id']) and (serializer.validated_data['b_prefix'] == True):
        #    serializer.validated_data['b_prefix'] = True
        instance = serializer.save()
        
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class PrefixListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = PrefixSerializer
    #
    queryset = Prefix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_prefix = Prefix.objects.filter(boilerplate = group_pk)
        return queryset_prefix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class PrefixDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = PrefixSerializer
    #queryset = Prefix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_prefix = Prefix.objects.filter(boilerplate = group_pk)
        return queryset_prefix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class LogicalExpressionListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = LogicalExpressionSerializer
    #queryset = LogicalExpression.objects.all()

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        pr = self.kwargs['prefixs']
        queryset_prefix = LogicalExpression.objects.filter(prefix_boilerplate = bp, prefix = pr)
        return queryset_prefix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class LogicalExpressionDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = LogicalExpressionSerializer
    queryset = LogicalExpression.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class OccuringFunctionalityP2ListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = OccuringFunctionalityP2Serializer
    #queryset = LogicalExpression.objects.all()

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        pr = self.kwargs['prefixs']

        queryset_prefix = OccuringFunctionalityP2.objects.filter(prefix_boilerplate_of_occuring_functionality_p2 = bp, prefix_of_occuring_functionality_p2 = pr)
        return queryset_prefix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class OccuringFunctionalityP2DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = OccuringFunctionalityP2Serializer
    queryset = OccuringFunctionalityP2.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()



class StateValueConstraintListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = StateValueConstraintSerializer
    #queryset = StateValueConstraint.objects.all()

    def get_queryset(self):
        le = self.kwargs['log_expr']
        pr = self.kwargs['prefixs']
        queryset_prefix = StateValueConstraint.objects.filter(logical_expression = le, prefix_of_logical_expression = pr)
        return queryset_prefix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class StateValueConstraintDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = StateValueConstraintSerializer
    queryset = StateValueConstraint.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class OccuringFunctionalityForLogicalExpressionListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = OccuringFunctionalityForLogicalExpressionSerializer
    #queryset = StateValueConstraint.objects.all()

    def get_queryset(self):
        le = self.kwargs['log_expr']
        pr = self.kwargs['prefixs']
        queryset_prefix = OccuringFunctionalityForLogicalExpression.objects.filter(logical_expression_of_occuring = le, prefix_of_occuring_for_logical_expression = pr)
        return queryset_prefix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class OccuringFunctionalityForLogicalExpressionDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = OccuringFunctionalityForLogicalExpressionSerializer
    queryset = OccuringFunctionalityForLogicalExpression.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()