from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models import Snippet 
from reqman.apps.reqtool.rest_api.serializers import SnippetSerializer
from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

class SnippetListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = SnippetSerializer
    queryset = Snippet.objects.all()

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)
        #print the data of the post
        write_fuseki = fuseki.FusekiActions()
        write_fuseki.write(serializer.data)

class SnippetDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly,
                      IsOwnerOrReadOnly]
    serializer_class = SnippetSerializer
    queryset = Snippet.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()
        write_fuseki = fuseki.FusekiActions()
        write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        print("deleted")
        instance.delete()